unit formPTChart;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes,
  Vcl.Graphics, Vcl.Controls, Vcl.Forms, Vcl.Dialogs,  Data.DB,
  Vcl.ExtCtrls,Vcl.StdCtrls, Vcl.buttons,  System.StrUtils, system.Math,
  VCLTee.DBChart, VCLTee.Chart, VCLTee.Series, VCLTee.TeeDownSampling,
  VclTee.TeeGDIPlus,  VCLTee.TeeMagnifyTool, VCLTee.TeEngine, VCLTee.TeeProcs,
  VCLTee.TeeScrollPagerTool, VCLTee.TeeSeriesTextEd, VCLTee.TeeTools, VCLTee.TeeURL;

type
  TfrmPTChart = class(TForm)
    DBChart1: TDBChart;
    Series1: TFastLineSeries;
    Series2: TFastLineSeries;
    Series3: TFastLineSeries;
    SeriesTextSource1: TSeriesTextSource;
    SeriesTextSource2: TSeriesTextSource;
    SeriesTextSource3: TSeriesTextSource;
    cbxSyncDep: TCheckBox;
    ChartTool1: TAxisScrollTool;
    ChartTool2: TAxisScrollTool;
    ChartTool3: TAxisScrollTool;
    ChartTool4: TAxisScrollTool;
    Panel1: TPanel;
    cbxAutoTop: TCheckBox;
    cbxAutoBottom: TCheckBox;
    CheckBox1: TCheckBox;

    procedure FormShow(Sender: TObject);
    procedure cbxSyncDepClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure DBChart1Scroll(Sender: TObject);
    procedure DBChart1UndoZoom(Sender: TObject);
    procedure cbxAutoBottomClick(Sender: TObject);
    procedure cbxAutoTopClick(Sender: TObject);
    procedure DBChart1Zoom(Sender: TObject);
    procedure CalcTolerance;
    procedure CheckBox1Click(Sender: TObject);

  private
    bOrigVisible: boolean;
    iWidth: integer;
    TopMin, TopMax, BotMin, BotMax: double;
    dSeries1, dSeries2, dSeries3: TFastLineSeries;
    dFunc: TDownSamplingFunction;
    minSector: Integer;
    procedure SyncTopAxis;

    { Private declarations }
  public
    { Public declarations }
  end;

var
  frmPTChart: TfrmPTChart;
  AVal: array of double;
  bFirstTime: boolean;

implementation

{$R *.dfm}

// ========== TfrmPTChart.FormCreate ===========================================
procedure TfrmPTChart.FormCreate(Sender: TObject);
begin
  FormatSettings.DecimalSeparator:='.';
  FormatSettings.ThousandSeparator:=',';
end;

// ========== TfrmPTChart.FormShow =============================================
procedure TfrmPTChart.FormShow(Sender: TObject);
var
  i: integer;
  LStrList: TStringList;

begin
  iWidth := 2;
  bOrigVisible := True;

// Load Pressure vs Depth (Left Axis)
  Series1.DataSource := SeriesTextSource1;
  Series1.VertAxis := aBothVertAxis;
  Series1.Pen.Style := psDash;
  Series1.Pen.Width := iWidth;


  with SeriesTextSource1 do
  begin
    FileName := 'C:\Code_Test_Progs\DownSampling2\DeviatedPT.txt';
    FieldSeparator:=',';
    Fields.Clear;
    AddField('X',3);
    AddField('Y',1);
    Load;
  end;

// Load WHP vs Depth (Left Axis)
  Series2.DataSource := SeriesTextSource2;
  Series2.VertAxis := aBothVertAxis;
  Series2.Pen.Style := psDot;
  Series2.Pen.Width := iWidth;

  with SeriesTextSource2 do
  begin
    FileName := 'C:\Code_Test_Progs\Sync_Axes\DeviatedPT.txt';
    FieldSeparator:=',';
    Fields.Clear;
    AddField('X',4);
    AddField('Y',1);
    Load;
  end;

// Load Temperature vs Depth (Left Axis)
  Series3.DataSource := SeriesTextSource3;
  Series3.VertAxis := aBothVertAxis;
  Series3.Pen.Style := psDashDot;
  Series3.Pen.Width := iWidth;

  with SeriesTextSource3 do
  begin
    FileName := 'C:\Code_Test_Progs\Sync_Axes\DeviatedPT.txt';
    FieldSeparator:=',';
    Fields.Clear;
    AddField('X',2);
    AddField('Y',1);
    Load;
  end;

  BotMin := 0;
  BotMax := 110;
  TopMin := 0;
  TopMax := 350;

  DBChart1.Axes.Bottom.Automatic := False;
  DBChart1.Axes.Bottom.SetMinMax(BotMin, BotMax);
  DBChart1.Axes.Bottom.Increment := 20;
  DBChart1.Axes.Top.Automatic := False;
  DBChart1.Axes.Top.SetMinMax(TopMin, TopMax);
  DBChart1.Axes.Top.Increment := 50;

  DBChart1.Axes.Right.Visible := True;
  DBChart1.Axes.Right.Labels := False;
  DBChart1.Axes.Top.MinorTickCount := 0;
  DBChart1.Axes.Bottom.MinorTickCount := 0;

  dSeries1 := DBChart1.AddSeries(TFastLineSeries) as TFastLineSeries;
  dSeries2 := DBChart1.AddSeries(TFastLineSeries) as TFastLineSeries;
  dSeries3 := DBChart1.AddSeries(TFastLineSeries) as TFastLineSeries;

  dFunc := TDownSamplingFunction.Create(Self);
  dFunc.DownSampleMethod := dsAverage;

  dSeries1.SetFunction(dFunc);
  dSeries1.Pen.Style := Series1.Pen.Style;
  dSeries1.Pen.Width := Series1.Pen.Width;
  dSeries1.Color := Series1.Color;
  dSeries1.HorizAxis := Series1.HorizAxis;
  dSeries1.DataSource := Series1;
  dSeries1.CheckDataSource;

  dSeries2.SetFunction(dFunc);
  dSeries2.Pen.Style := Series2.Pen.Style;
  dSeries2.Pen.Width := Series2.Pen.Width;
  dSeries2.Color := Series2.Color;
  dSeries2.HorizAxis := Series2.HorizAxis;
  dSeries2.DataSource := Series2;
  dSeries2.CheckDataSource;

  dSeries3.SetFunction(dFunc);
  dSeries3.Pen.Style := Series3.Pen.Style;
  dSeries3.Pen.Width := Series3.Pen.Width;
  dSeries3.Color := Series3.Color;
  dSeries3.HorizAxis := Series3.HorizAxis;
  dSeries3.DataSource := Series3;
  dSeries3.CheckDataSource;

  Series1.Visible := bOrigVisible;
  Series2.Visible := bOrigVisible;
  Series3.Visible := bOrigVisible;
  DBChart1.OnZoom := DBChart1Zoom;
  DBChart1.OnUndoZoom := DBChart1UndoZoom;

  CalcTolerance;

end;

// ========== TfrmPTChart.cbxSyncDepClick ======================================
// Doesn't work very well
procedure TfrmPTChart.cbxSyncDepClick(Sender: TObject);
var
  iInc: integer;

begin
  if cbxSyncDep.Checked then
    SyncTopAxis
  else
//if not cbxSyncDep.Checked then

  begin
    DBChart1.Axes.Top.Items.Automatic := True;
  end;

end;

procedure TfrmPTChart.CheckBox1Click(Sender: TObject);
begin
  bOrigVisible := not CheckBox1.Checked;
  if not CheckBox1.Checked then
    CheckBox1.Caption := 'Hide Original Series'
  else
    CheckBox1.Caption := 'Show Original Series';
  Series1.Visible := bOrigVisible;
  Series2.Visible := bOrigVisible;
  Series3.Visible := bOrigVisible;
  DBChart1.Draw;
end;

// ========== TfrmPTChart.cbxAutoBottomClick ===================================
procedure TfrmPTChart.cbxAutoBottomClick(Sender: TObject);
begin
  DBChart1.Axes.Bottom.Automatic := cbxAutoBottom.Checked;

  if DBChart1.Axes.Bottom.Automatic then
    DBChart1.Axes.Bottom.Increment := 0
  else
    DBChart1.Axes.Bottom.SetMinMax(TopMin, TopMax);

  if cbxSyncDep.Checked then
  begin
    DBChart1.Draw;
    SyncTopAxis;
  end;

end;

// ========== TfrmPTChart.cbxAutoTopClick ======================================
procedure TfrmPTChart.cbxAutoTopClick(Sender: TObject);
begin
  DBChart1.Axes.Top.Automatic := cbxAutoTop.Checked;

  if DBChart1.Axes.Top.Automatic then
    DBChart1.Axes.Top.Increment := 0
  else
    DBChart1.Axes.Top.SetMinMax(TopMin, TopMax);

  if cbxSyncDep.Checked then
  begin
    DBChart1.Draw;
    SyncTopAxis;
  end;

end;

// ========== TfrmPTChart.DBChart1Scroll =======================================
procedure TfrmPTChart.DBChart1Scroll(Sender: TObject);
var
  j: integer;

begin
  if cbxSyncDep.Checked then
    SyncTopAxis;
end;

// ========== TfrmPTChart.DBChart1UndoZoom =====================================
procedure TfrmPTChart.DBChart1UndoZoom(Sender: TObject);
begin
  if cbxSyncDep.Checked then
    SyncTopAxis;
  CalcTolerance;
end;

// ========== TfrmPTChart.DBChart1Zoom =========================================
procedure TfrmPTChart.DBChart1Zoom(Sender: TObject);
begin
  if cbxSyncDep.Checked then
    SyncTopAxis;
  CalcTolerance;
end;

// ========== TfrmPTChart.SyncTopAxis ==========================================
// Uses bottom axis increments to locate top axis labels
procedure TfrmPTChart.SyncTopAxis;
var
  i, iMin, iMax, Factor: integer;
  d: double;
  Tval, TPos, TMin, TMax, TDiff, BMin, BMax, BDiff, BInc: double;

begin
  Factor := 1;

  DBChart1.Draw;
  DBChart1.Axes.Top.Items.Clear;

  TMin := DBChart1.Axes.Top.Minimum;
  TMax := DBChart1.Axes.Top.Maximum;
  TDiff := TMax - Tmin;

  BMin := DBChart1.Axes.Bottom.Minimum;
  BMax := DBChart1.Axes.Bottom.Maximum;
  BDiff := BMax - BMin;

  BInc := DBChart1.Axes.Bottom.CalcIncrement;

  if (BInc * TDiff/BDiff < 0.01)  then
    Factor := 1000
  else if (BInc * TDiff/BDiff < 0.1) then
    Factor := 100
  else if (BInc * TDiff/BDiff < 1) then
    Factor := 10;

  iMax := DBChart1.Axes.Bottom.Items.Count;
  for i := 0 to iMax - 1 do
  begin
    Tval := Round(((DBChart1.Axes.Bottom.Items[iMax - i - 1].Value - BMin)/BDiff*TDiff + TMin)*Factor)/Factor;
    TPos := (DBChart1.Axes.Bottom.Items[iMax - i - 1].Value - BMin)/BDiff*TDiff + TMin;
    DBChart1.Axes.Top.Items.Add(TPos, FloatToStr(Tval));  // adding custom label
  end;
end;

procedure TfrmPTChart.CalcTolerance;
var
  iRight, iLeft, iLast, iFirst: integer;

begin
  dFunc.Tolerance:=1;
  DBChart1.Draw;

  if dSeries1.Count>0 then
  begin
    iRight := DBChart1.ChartRect.Right;
    iLeft := DBChart1.ChartRect.Left;
    iLast := dSeries1.LastDisplayedIndex;
    iFirst := dSeries1.FirstDisplayedIndex;

//    while (DBChart1.ChartRect.Right-DBChart1.ChartRect.Left) div (dSeries1.LastDisplayedIndex-dSeries1.FirstDisplayedIndex) < 5 do
    while (iRight-iLeft) div (iLast - iFirst) < 5 do
    begin
      dFunc.Tolerance:=dFunc.Tolerance+1;
      dSeries1.CalcFirstLastVisibleIndex;
      iRight := DBChart1.ChartRect.Right;
      iLeft := DBChart1.ChartRect.Left;
      iLast := dSeries1.LastDisplayedIndex;
      iFirst := dSeries1.FirstDisplayedIndex;
    end;
  end;

  Caption:=FormatFloat('#,##0', dFunc.Tolerance);
end;

end.
